﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Input;
using System.Windows.Controls;
using Telerik.Windows.Controls;
#if SILVERLIGHT
using Telerik.Windows;
using ContextMenu = Telerik.Windows.Controls.RadContextMenu;
#endif

namespace HierarchyConverter.Wpf.Sample
{
	public class ContextMenuBuilder
	{
#if SILVERLIGHT
		public static Telerik.Windows.RoutedEvent BuildContextMenu =
#else
		public static RoutedEvent BuildContextMenu = 
#endif
 EventManager.RegisterRoutedEvent("BuildContextMenu", RoutingStrategy.Bubble, typeof(BuildContextMenuEventHandler), typeof(ContextMenuBuilder));

		static ContextMenuBuilder()
		{
			EventManager.RegisterClassHandler(typeof(Control), BuildContextMenu, new BuildContextMenuEventHandler(OnControlBuildContextMenu));
		}

		private static void OnControlBuildContextMenu(object sender, BuildContextMenuEventArgs e)
		{
			var element = sender as Control;
			var contextMenuAware = element.DataContext as IContextMenuProvider;
			if (contextMenuAware != null)
			{
				foreach (var command in contextMenuAware.ProvideCommands())
				{
					if (!e.CommandsAndParameters.ContainsKey(command))
					{
						e.CommandsAndParameters[command] = contextMenuAware;
					}
				}
				e.Handled = true;
			}
		}

#if SILVERLIGHT
		internal static void OpenContextMenu(RadContextMenu contextMenu, UIElement sourceUIElement, Point p)
		{
			contextMenu.Placement = PlacementMode.Absolute;
			contextMenu.PlacementRectangle = new Rect(p, new Size());
			contextMenu.Items.Clear();

			var args = new BuildContextMenuEventArgs(sourceUIElement);
			sourceUIElement.RaiseEvent(args);

			var menuItemTemplate = Application.Current.Resources["AppContextMenuItemTemplate"] as DataTemplate;

			foreach (var pair in args.CommandsAndParameters)
			{
				var menuItem = new RadMenuItem();
				menuItem.Header = pair.Key;
				menuItem.CommandParameter = pair.Value;
				menuItem.Command = pair.Key;
				menuItem.HeaderTemplate = menuItemTemplate;
				contextMenu.Items.Add(menuItem);
			}

			contextMenu.IsOpen = contextMenu.HasItems;
		}
	}
#else
		internal static void OpenContextMenu(UIElement sourceUIElement)
		{
			var contextMenu = new ContextMenu();
			contextMenu.PlacementTarget = sourceUIElement;
			contextMenu.Placement = System.Windows.Controls.Primitives.PlacementMode.MousePoint;
			var args = new BuildContextMenuEventArgs(sourceUIElement);
			sourceUIElement.RaiseEvent(args);

			var menuItemTemplate = App.Current.Resources["AppContextMenuItemTemplate"] as DataTemplate;

			foreach(var pair in args.CommandsAndParameters)
			{
				var menuItem = new MenuItem();

				contextMenu.Items.Add(menuItem);

				menuItem.Header = pair.Key;
				menuItem.CommandParameter = pair.Value;
				menuItem.Command = pair.Key;
				menuItem.HeaderTemplate = menuItemTemplate;
			}

			contextMenu.IsOpen = true;

			contextMenu.Dispatcher.BeginInvoke(new Action(() =>
				{
					CommandManager.InvalidateRequerySuggested();
					foreach (MenuItem menuItem in contextMenu.Items)
					{
					}
				}));
		}
	}
#endif

#if SILVERLIGHT
	public class BuildContextMenuEventArgs : RadRoutedEventArgs
#else
	public class BuildContextMenuEventArgs:RoutedEventArgs
#endif
	{
		public BuildContextMenuEventArgs(object source)
			: base(ContextMenuBuilder.BuildContextMenu, source)
		{
			CommandsAndParameters = new Dictionary<ICommand, object>();
		}

		public IDictionary<ICommand, object> CommandsAndParameters
		{
			get;
			set;
		}
	}

	public interface IContextMenuProvider
	{
		IEnumerable<ICommand> ProvideCommands();
	}

	public delegate void BuildContextMenuEventHandler(object sender, BuildContextMenuEventArgs e);

	public class DelegateCommand<T> : ICommand
	{
		Action<T> onExecute;
		Func<T, bool> onCanExecute;

		public DelegateCommand(string text, Action<T> onExecute, Func<T, bool> onCanExecute)
		{
			this.onCanExecute = onCanExecute;
			this.onExecute = onExecute;
			this.Text = text;
		}

		public string Text { get; set; }

		public bool CanExecute(object parameter)
		{
			if (onCanExecute != null && parameter is T)
			{
				return onCanExecute((T)parameter);
			}
			return true;
		}

		public event EventHandler CanExecuteChanged;

		public void Execute(object parameter)
		{
			if (onExecute != null && parameter is T)
			{
				onExecute((T)parameter);
			}
		}
	}
}
