﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Linq;

namespace TreeViewContextMenu.ViewModels
{
	public abstract class DiskItemBase : INotifyPropertyChanged
	{
		private bool isEdited;
		private static char[] invalidChars = new char[] { '/', '\\', '<', '>', '|', ':', '?', '*' };

		protected void SetProperty<T>(string propertyName, ref T field, T value)
		{
			if (!object.Equals(field, value))
			{
				field = value;
				this.OnPropertyChanged(propertyName);
			}
		}

		protected void ValidateName(string name)
		{
			if (String.IsNullOrEmpty(name) || String.IsNullOrEmpty(name.Trim()))
			{
				throw new Exception("Please enter a name");
			}
			if (name.IndexOfAny(invalidChars) != -1)
			{
				throw new Exception(String.Format("A file name cannot contain any of the following characters: \n {0}", String.Join(", ", invalidChars.Select(c => c.ToString()).ToArray())));
			}
		}

		/// <summary>
		///     Event for INotifyPropertyChanged.
		/// </summary>
		public event PropertyChangedEventHandler PropertyChanged;

		/// <summary>
		///     Raises the INotifyPropertyChanged event.
		/// </summary>
		/// <param name="propertyName">The property to be included in the INotifyPropertyChanged EventArgs.</param>
		protected virtual void OnPropertyChanged(String propertyName)
		{
			if (PropertyChanged != null)
			{
				PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}

		public bool IsEdited
		{
			get
			{
				return this.isEdited;
			}
			set
			{
				this.SetProperty("IsEdited", ref this.isEdited, value);
			}
		}

		public abstract string TypeName
		{
			get;
		}

		public abstract string Name
		{
			get;
			set;
		}
	}
}
