﻿using System;
using System.Collections;
using System.Collections.ObjectModel;
using HierarchyConverter.Wpf.Sample;
using System.Collections.Generic;
using System.Windows.Input;

namespace TreeViewContextMenu.ViewModels
{
	

	/// <summary>
	/// This class is a ViewModel for a file folder.
	/// </summary>
	public class FolderViewModel : DiskItemBase, IContextMenuProvider
	{
		public static readonly Random Generator = new Random(12431231);
		private static readonly int maxLevels = 5;
		private static readonly int maxChildren = 20;
		private static readonly double percentPeople = 0.7;

		private static readonly string[] adjectives;
		private static readonly string[] nouns;

		static FolderViewModel()
		{
			adjectives = @"Sales IT Support Admin Administration 
			Management Research QA Quality Shipping 
			Installer Upgrades Fixes Planning Testing
			New Secret Secretive Security Finace
			Ecology Environment".Split(new char[] { ' ', '\n', '\t', '\r' }, StringSplitOptions.RemoveEmptyEntries);

			nouns = "Team Department Branch Division Group".Split(new char[] { ' ', '\n', '\t', '\r' }, StringSplitOptions.RemoveEmptyEntries);
		}

		private double level;
		private string name;
		private int number;
		private IList children;

		public static int UidCount { get; set; }

		public override string TypeName
		{
			get
			{
				return "File Folder";
			}
		}

		public FolderViewModel(int number, double level)
		{
			this.number = number;
			this.level = level;
			this.Uid = UidCount++;
		}

		public override string Name
		{
			get
			{
				if (this.name == null)
				{
					this.name = BuildName();
				}
				return name;
			}
			set
			{
				this.ValidateName(value);
				this.SetProperty("Name", ref this.name, value);
			}
		}

		public int Uid { get; set; }

		private string BuildName()
		{
			return string.Format("{0} {1}", adjectives[Generator.Next(adjectives.Length)], nouns[Generator.Next(nouns.Length)]);
		}

		public IList Children
		{
			get
			{
				if (this.children == null)
				{
					this.children = BuildChildren();
				}
				return this.children;
			}
		}

		private IList BuildChildren()
		{
			//Make sure that each level has less and less items:
			double childrenCount = Generator.Next(maxChildren) * (1 - this.level / maxLevels);
			int peopleCount = Convert.ToInt32(childrenCount * percentPeople);
			int subOrganisationsCount = Convert.ToInt32(childrenCount * (1 - percentPeople));
			double childrenLevel = this.level + 1;

			ObservableCollection<object> result = new ObservableCollection<object>();

			for (int orgIndex = 0; orgIndex < subOrganisationsCount; orgIndex++)
			{
				result.Add(new FolderViewModel(orgIndex, childrenLevel));
			}

			for (int personIndex = 0; personIndex < peopleCount; personIndex++)
			{
				result.Add(new FileViewModel());
			}

			return result;
		}

		public static ICommand RenameFolderCommand = new DelegateCommand<FolderViewModel>(
			"Rename",
			f => f.IsEdited = true,
			null);

		public static ICommand NewFileCommand = new DelegateCommand<FolderViewModel>(
			"New File",
			f => f.Children.Add(new FileViewModel()),
			null);

		public static ICommand NewFolderCommand = new DelegateCommand<FolderViewModel>(
			"New Folder",
			f => f.Children.Add(new FolderViewModel(0, 100)),
			null);

		public IEnumerable<ICommand> ProvideCommands()
		{
			yield return RenameFolderCommand;
			yield return NewFolderCommand;
			yield return NewFileCommand;
		}
	}
}
