﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.Windows.Input;
using System.Windows.Threading;

namespace HierarchyConverter.Wpf.Sample.ViewModels
{
	public class FolderViewModel : ViewModelBase, IContextMenuProvider, IHierarchyAware
	{
		private static readonly string[] adjectives;
		private static readonly string[] nouns;
		public static readonly Random Generator = new Random(12431231);

		static FolderViewModel()
		{
			adjectives = @"Sales IT Support Admin Administration 
			Management Research QA Quality Shipping 
			Installer Upgrades Fixes Planning Testing
			New Secret Secretive Security Finace
			Ecology Environment".Split(new char[] { ' ', '\n', '\t', '\r' }, StringSplitOptions.RemoveEmptyEntries);

			nouns = "Team Department Branch Division Group".Split(new char[] { ' ', '\n', '\t', '\r' }, StringSplitOptions.RemoveEmptyEntries);
		}

		private static string BuildName()
		{
			return string.Format("{0} {1}", adjectives[Generator.Next(adjectives.Length)], nouns[Generator.Next(nouns.Length)]);
		}

		public FolderViewModel()
		{
			timer.Tick += new EventHandler(timer_Tick);
		}

		void timer_Tick(object sender, EventArgs e)
		{
			this.Id = ++idCounter;
			timer.Stop();
		}

		private static int idCounter;

		private static Random generator = new Random(1231);
		public static IList<FolderViewModel> Generate()
		{
			var result = new ObservableCollection<FolderViewModel>();
			for (idCounter = 1; idCounter < 200; idCounter++)
			{
				var newModel = new FolderViewModel();
				newModel.Name = BuildName();
				newModel.Id = idCounter;
				result.Add(newModel);
			}

			// Add a few root items:
			for (var i = 1; i < 10; i++)
			{
				result[i].ParentId = 0;
			}

			// Assign parent IDs:
			for (var i = 10; i < 199; i++)
			{
				result[i].ParentId = result[generator.Next(i)].Id;
			}

			return result;
		}


		private bool isInEditMode;
		/// <summary>
		///     Gets or sets the name.
		/// </summary>
		public bool IsInEditMode
		{
			get
			{
				return this.isInEditMode;
			}
			set
			{
				this.SetProperty("IsInEditMode", ref isInEditMode, value);
				this.OnIsinEditModeChanged(value);
			}
		}

		DispatcherTimer timer = new DispatcherTimer() { Interval = TimeSpan.FromSeconds(3) };

		private void OnIsinEditModeChanged(bool value)
		{
			if (!value && IsTempId(this.Id))
			{
				//Brand new item, needs to be persisted
				timer.Start();
			}
		}


		private string name;
		/// <summary>
		///     Gets or sets the name.
		/// </summary>
		public string Name
		{
			get
			{
				return this.name;
			}
			set
			{
				this.SetProperty("Name", ref name, value);
			}
		}


		private int id;
		/// <summary>
		///     Gets or sets the id of the folder.
		/// </summary>
		public int Id
		{
			get
			{
				return this.id;
			}
			set
			{
				this.SetProperty("Id", ref id, value);
			}
		}


		private int parentId;
		/// <summary>
		///     Gets or sets the id of the parent folder.
		/// </summary>
		public int ParentId
		{
			get
			{
				return this.parentId;
			}
			set
			{
				this.SetProperty("ParentId", ref parentId, value);
			}
		}

		private bool isExpanded;
		/// <summary>
		///     Gets or sets whther the item is expanded.
		/// </summary>
		public bool IsExpanded
		{
			get
			{
				return this.isExpanded;
			}
			set
			{
				this.SetProperty("IsExpanded", ref isExpanded, value);
			}
		}



		public static DelegateCommand<FolderViewModel> Remove = new DelegateCommand<FolderViewModel>("Unload", OnRemoveExecute, null);

		private static void OnRemoveExecute(FolderViewModel folder)
		{
			folder.ParentId = -1;
		}

		public static DelegateCommand<FolderViewModel> AddChild = new DelegateCommand<FolderViewModel>("Add New Item", OnAddChildExecute, null);

		public static IList<ICommand> Commands = new List<ICommand>();

		private static int safeIdCounter = int.MaxValue - 10000;

		private static void OnAddChildExecute(FolderViewModel folder)
		{
			var child = new FolderViewModel();
			child.Id = GetTempId();
			child.Name = "New Item";
			child.IsInEditMode = true;

			folder.children.Add(child);
			folder.IsExpanded = true;
		}

		private bool IsTempId(int id)
		{
			return id >= int.MaxValue - 10000; 
		}

		private static int GetTempId()
		{
			return safeIdCounter++;
		}

		public IEnumerable<System.Windows.Input.ICommand> ProvideCommands()
		{
			yield return AddChild;
			yield return Remove;
		}

		IList<FolderViewModel> children;

		public void SetChildren(System.Collections.IEnumerable children, object converter)
		{
			this.children = children as IList<FolderViewModel>;
		}
	}
}
